@@ -294,7 +294,9 @@ module RubyIndexer
 
       # @abstract
       #: -> Array[Signature]
-      def signatures; end
+      def signatures
+        raise AbstractMethodInvokedError
+      end
 
       #: -> String
       def decorated_parameters
@@ -331,6 +333,7 @@ module RubyIndexer
     end
 
     class Method < Member
+      # @override
       #: Array[Signature]
       attr_reader :signatures
 
