@@ -3,14 +3,13 @@
 
 module RubyLsp
   module ResponseBuilders
+    # @abstract
     class ResponseBuilder
-      extend T::Generic
-      extend T::Sig
-
-      abstract!
-
-      sig { abstract.returns(T.anything) }
-      def response; end
+      # @abstract
+      #: -> top
+      def response
+        raise AbstractMethodInvokedError
+      end
     end
   end
 end
