@@ -5,7 +5,6 @@ module RubyLsp
   # rubocop:disable RubyLsp/UseLanguageServerAliases
   Interface = LanguageServer::Protocol::Interface
   Constant = LanguageServer::Protocol::Constant
-  Transport = LanguageServer::Protocol::Transport
   # rubocop:enable RubyLsp/UseLanguageServerAliases
 
   # Used to indicate that a request shouldn't return a response
@@ -20,6 +19,7 @@ module RubyLsp
     "Gemfile"
   end #: String
   GUESSED_TYPES_URL = "https://shopify.github.io/ruby-lsp/#guessed-types"
+  TEST_PATH_PATTERN = "**/{test,spec,features}/**/{*_test.rb,test_*.rb,*_spec.rb,*.feature}"
 
   # Request delegation for embedded languages is not yet standardized into the language server specification. Here we
   # use this custom error class as a way to return a signal to the client that the request should be delegated to the
@@ -31,6 +31,8 @@ module RubyLsp
     CODE = -32900
   end
 
+  class AbstractMethodInvokedError < StandardError; end
+
   BUNDLE_COMPOSE_FAILED_CODE = -33000
 
   # A notification to be sent to the client
@@ -50,7 +52,9 @@ module RubyLsp
 
     # @abstract
     #: -> Hash[Symbol, untyped]
-    def to_hash; end
+    def to_hash
+      raise AbstractMethodInvokedError
+    end
   end
 
   class Notification < Message
@@ -243,9 +247,6 @@ module RubyLsp
 
   # A request configuration, to turn on/off features
   class RequestConfig
-    #: Hash[Symbol, bool]
-    attr_accessor :configuration
-
     #: (Hash[Symbol, bool] configuration) -> void
     def initialize(configuration)
       @configuration = configuration
@@ -255,6 +256,11 @@ module RubyLsp
     def enabled?(feature)
       @configuration[:enableAll] || @configuration[feature]
     end
+
+    #: (Hash[Symbol, bool]) -> void
+    def merge!(hash)
+      @configuration.merge!(hash)
+    end
   end
 
   class SorbetLevel
@@ -299,4 +305,37 @@ module RubyLsp
     #: -> bool
     def true_or_higher? = @level == :true || @level == :strict
   end
+
+  # Reads JSON RPC messages from the given IO in a loop
+  class MessageReader
+    #: (IO) -> void
+    def initialize(io)
+      @io = io
+    end
+
+    #: () { (Hash[Symbol, untyped]) -> void } -> void
+    def each_message(&block)
+      while (headers = @io.gets("\r\n\r\n"))
+        raw_message = @io.read(headers[/Content-Length: (\d+)/i, 1].to_i) #: as !nil
+        block.call(JSON.parse(raw_message, symbolize_names: true))
+      end
+    end
+  end
+
+  # Writes JSON RPC messages to the given IO
+  class MessageWriter
+    #: (IO) -> void
+    def initialize(io)
+      @io = io
+    end
+
+    #: (Hash[Symbol, untyped]) -> void
+    def write(message)
+      message[:jsonrpc] = "2.0"
+      json_message = message.to_json
+
+      @io.write("Content-Length: #{json_message.bytesize}\r\n\r\n#{json_message}")
+      @io.flush
+    end
+  end
 end
